﻿using JianGongYun.TRTC.Models;
using JianGongYun.TRTC.Utils;
using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.ComponentModel;
using System.Text;
using System.Linq;
using System.Windows.Controls;
using JianGongYun.TRTC.Components;
using System.Windows;

namespace JianGongYun.TRTC.ViewModels
{
    public class SettingWindowViewModel : INotifyPropertyChanged
    {
        public event PropertyChangedEventHandler PropertyChanged;
        /// <summary>
        /// 单例
        /// </summary>
        private static SettingWindowViewModel _Instance;

        public static SettingWindowViewModel GetInstance()
        {
            if (_Instance == null)
            {
                _Instance = new SettingWindowViewModel();
            }
            return _Instance;
        }

        /// <summary>
        /// 直播主窗口viewmodel
        /// </summary>
        private LiveWindowViewModel LiveWindowViewModel
        {
            get
            {
                return LiveClassroom.CurrentLiveWindow.DataContext == null ? null : LiveClassroom.CurrentLiveWindow.DataContext as LiveWindowViewModel;
            }
        }

        #region 麦克风
        /// <summary>
        /// 集合，实时获取
        /// </summary>
        public List<ComboBoxEntity> MicList
        {
            get
            {
                var mics = LiveClassroom.lTXDeviceManager.getDevicesList(ManageLiteAV.TRTCDeviceType.TXMediaDeviceTypeMic);
                List<ComboBoxEntity> micList = new List<ComboBoxEntity>();
                var count = mics.getCount();
                for (uint i = 0; i < count; i++)
                {
                    var temp = new ComboBoxEntity { Id = mics.getDevicePID(i), Text = mics.getDeviceName(i) };
                    micList.Add(temp);
                }
                return micList;
            }
        }

        /// <summary>
        /// 当前
        /// </summary>
        private string _CurrentMic;
        public string CurrentMic
        {
            get
            {
                var tempList = this.MicList;
                if (tempList.Count == 0)
                {
                    return null;
                }
                if (string.IsNullOrEmpty(_CurrentMic) || !tempList.Any(a => a.Id == _CurrentMic))//第一次取或者设备发生变动找不到上次的选择
                {
                    var localMic = storage.GetValue(INI_ROOT_KEY, INI_KEY_CHOOSE_MIC);//本地缓存
                    if (string.IsNullOrEmpty(localMic))//本地没有就取第一个
                    {
                        _CurrentMic = tempList.First().Id;
                        storage.SetValue(INI_ROOT_KEY, INI_KEY_CHOOSE_MIC, _CurrentMic);//生成初始数据
                    }
                    else
                    {
                        _CurrentMic = localMic;
                        var temp = tempList.Where(a => a.Id == localMic).FirstOrDefault();
                        if (temp == null)//找不到上次缓存的设备就取第一个
                        {
                            _CurrentMic = tempList.First().Id;
                            storage.SetValue(INI_ROOT_KEY, INI_KEY_CHOOSE_MIC, _CurrentMic);//生成初始数据
                        }
                    }
                }
                return _CurrentMic;
            }
            set
            {
                _CurrentMic = value;
                storage.SetValue(INI_ROOT_KEY, INI_KEY_CHOOSE_MIC, value);//生成保存本地
                LiveClassroom.lTXDeviceManager.setCurrentDevice(ManageLiteAV.TRTCDeviceType.TXMediaDeviceTypeMic, _CurrentMic);//设置麦克
                if (PropertyChanged != null)
                {
                    PropertyChanged(this, new PropertyChangedEventArgs("CurrentMic"));//对CurrentMic进行监听
                }
            }
        }


        /// <summary>
        /// 麦克风采集音量
        /// </summary>
        private uint _MicVolume = 100;
        public uint MicVolume
        {
            get
            {
                var micVolume = storage.GetValue(INI_ROOT_KEY, INI_KEY_AUDIO_MIC_VOLUME);//本地缓存
                if (string.IsNullOrEmpty(micVolume))//本地没有就取第一个
                {
                    storage.SetValue(INI_ROOT_KEY, INI_KEY_AUDIO_MIC_VOLUME, _MicVolume.ToString());//生成初始数据
                }
                else
                {
                    _MicVolume = uint.Parse(micVolume);
                }
                return _MicVolume;
            }
            set
            {
                _MicVolume = value;
                storage.SetValue(INI_ROOT_KEY, INI_KEY_AUDIO_MIC_VOLUME, value.ToString());//生成保存本地
                var res = LiveClassroom.lTXDeviceManager.setCurrentDeviceVolume(ManageLiteAV.TRTCDeviceType.TXMediaDeviceTypeMic, _MicVolume);//设置麦克音量
                Console.WriteLine("设置麦克风采集音量" + res);
                if (PropertyChanged != null)
                {
                    PropertyChanged(this, new PropertyChangedEventArgs("MicVolume"));
                }
            }
        }


        #endregion

        #region 音频来源

        /// <summary>
        /// 音频来源
        /// </summary>
        private string _AudioSource = "1";
        public string AudioSource
        {
            get
            {
                var audioSource = storage.GetValue(INI_ROOT_KEY, INI_KEY_AUDIO_SOURCE);//本地缓存
                if (string.IsNullOrEmpty(audioSource))//本地没有就取第一个
                {
                    storage.SetValue(INI_ROOT_KEY, INI_KEY_AUDIO_SOURCE, _AudioSource);//生成初始数据
                }
                else
                {
                    _AudioSource = audioSource;
                }
                return _AudioSource;
            }
            set
            {
                _AudioSource = value;
                storage.SetValue(INI_ROOT_KEY, INI_KEY_AUDIO_SOURCE, value);//生成保存本地
                if (_AudioSource == "1")//只采集麦克风，关闭系统音采集
                {
                    ShowSytemGatherSlider = Visibility.Hidden;
                    LiveClassroom.lTRTCCloud.stopSystemAudioLoopback();
                }
                else if (_AudioSource == "2")//麦和系统音，开启采集系统音
                {
                    ShowSytemGatherSlider = Visibility.Visible;
                    LiveClassroom.lTRTCCloud.startSystemAudioLoopback(null);
                }
                if (PropertyChanged != null)
                {
                    PropertyChanged(this, new PropertyChangedEventArgs("AudioSource"));
                }
            }
        }

        /// <summary>
        /// 是否显示系统采集滑动条
        /// </summary>
        private Visibility _ShowSytemGatherSlider =  Visibility.Hidden;
        public Visibility ShowSytemGatherSlider
        {
            get
            {
                if (AudioSource=="2")
                {
                    _ShowSytemGatherSlider = Visibility.Visible;
                }
                else
                {
                    _ShowSytemGatherSlider = Visibility.Hidden;
                }
                return _ShowSytemGatherSlider;
            }
            set
            {
                _ShowSytemGatherSlider = value;
                if (PropertyChanged != null)
                {
                    PropertyChanged(this, new PropertyChangedEventArgs("ShowSytemGatherSlider"));
                }
            }
        }

        /// <summary>
        /// 系统采集音量
        /// </summary>
        private uint _SytemGatherVolume = 100;
        public uint SytemGatherVolume
        {
            get
            {
                var sytemGatherVolume = storage.GetValue(INI_ROOT_KEY, INI_KEY_AUDIO_SYSTEM_GATHER_VOLUME);//本地缓存
                if (string.IsNullOrEmpty(sytemGatherVolume))//本地没有就取第一个
                {
                    storage.SetValue(INI_ROOT_KEY, INI_KEY_AUDIO_SYSTEM_GATHER_VOLUME, _SytemGatherVolume.ToString());//生成初始数据
                }
                else
                {
                    _SytemGatherVolume = uint.Parse(sytemGatherVolume);
                }
                return _SytemGatherVolume;
            }
            set
            {
                _SytemGatherVolume = value;
                storage.SetValue(INI_ROOT_KEY, INI_KEY_AUDIO_SYSTEM_GATHER_VOLUME, value.ToString());//生成保存本地
                LiveClassroom.lTRTCCloud.setSystemAudioLoopbackVolume(_SytemGatherVolume);//设置系统采集音量
                if (PropertyChanged != null)
                {
                    PropertyChanged(this, new PropertyChangedEventArgs("SytemGatherVolume"));
                }
            }
        }


        #endregion


        /// <summary>
        /// ini文件操作
        /// </summary>
        private IniStorage storage;
        private SettingWindowViewModel()
        {
            storage = new IniStorage(sPath);
        }

        #region 配置持久化

        //配置文件路径
        private string sPath = ".\\TRTCConfig.ini";
        //根节点
        public const string INI_ROOT_KEY = "TRTCLOCALCONFIG";
        // 设备
        public const string INI_KEY_CHOOSE_CAMERA = "INI_KEY_CHOOSE_CAMERA";//当前摄像头
        public const string INI_KEY_CHOOSE_MIC = "INI_KEY_CHOOSE_MIC";//当前麦克风
        // 音频
        public const string INI_KEY_AUDIO_MIC_VOLUME = "INI_KEY_AUDIO_MIC_VOLUME";//麦克风音量
        public const string INI_KEY_AUDIO_SPEAKER_VOLUME = "INI_KEY_AUDIO_SOURCE";//播放音量
        public const string INI_KEY_AUDIO_SOURCE = "INI_KEY_AUDIO_SOURCE";//音频来源
        public const string INI_KEY_AUDIO_SYSTEM_GATHER_VOLUME = "INI_KEY_AUDIO_SYSTEM_GATHER_VOLUME";//麦克风音量
        // 视频
        public const string INI_KEY_VIDEO_FPS = "INI_KEY_VIDEO_FPS";//视频FPS
        public const string INI_KEY_VIDEO_BITRATE = "INI_KEY_VIDEO_BITRATE";
        public const string INI_KEY_VIDEO_RESOLUTION = "INI_KEY_VIDEO_RESOLUTION";
        public const string INI_KEY_VIDEO_RES_MODE = "INI_KEY_VIDEO_RES_MODE";
        public const string INI_KEY_VIDEO_QUALITY = "INI_KEY_VIDEO_QUALITY";
        public const string INI_KEY_VIDEO_QUALITY_CONTROL = "INI_KEY_VIDEO_QUALITY_CONTROL";
        public const string INI_KEY_VIDEO_APP_SCENE = "INI_KEY_VIDEO_APP_SCENE";
        public const string INI_KEY_VIDEO_FILL_MODE = "INI_KEY_VIDEO_FILL_MODE";
        public const string INI_KEY_VIDEO_ROTATION = "INI_KEY_VIDEO_ROTATION";
        // 美颜
        public const string INI_KEY_BEAUTY_OPEN = "INI_KEY_BEAUTY_OPEN";
        public const string INI_KEY_BEAUTY_STYLE = "INI_KEY_BEAUTY_STYLE";
        public const string INI_KEY_BEAUTY_VALUE = "INI_KEY_BEAUTY_VALUE";
        public const string INI_KEY_WHITE_VALUE = "INI_KEY_WHITE_VALUE";
        public const string INI_KEY_RUDDINESS_VALUE = "INI_KEY_RUDDINESS_VALUE";
        // 大小流
        public const string INI_KEY_SET_PUSH_SMALLVIDEO = "INI_KEY_SET_PUSH_SMALLVIDEO";
        public const string INI_KEY_SET_PLAY_SMALLVIDEO = "INI_KEY_SET_PLAY_SMALLVIDEO";
        // 测试
        public const string INI_KEY_SET_NETENV_STYLE = "INI_KEY_SET_NETENV_STYLE";
        public const string INI_KEY_ROOMCALL_STYLE = "INI_KEY_ROOMCALL_STYLE";
        // 镜像
        public const string INI_KEY_LOCAL_VIDEO_MIRROR = "INI_KEY_LOCAL_VIDEO_MIRROR";
        public const string INI_KEY_REMOTE_VIDEO_MIRROR = "INI_KEY_REMOTE_VIDEO_MIRROR";
        // 音量提示
        public const string INI_KEY_SHOW_AUDIO_VOLUME = "INI_KEY_SHOW_AUDIO_VOLUME";
        // 混流
        public const string INI_KEY_CLOUD_MIX_TRANSCODING = "INI_KEY_CLOUD_MIX_TRANSCODING";
        #endregion
    }
}
